/******************************************************************************
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Orignal Author: Pawel Sosulski <pawel@sosulski.net>
******************************************************************************/

const Clutter = imports.gi.Clutter;
const Gtk = imports.gi.Gtk;
const Gio = imports.gi.Gio;
const St = imports.gi.St;
const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;
const PanelMenu = imports.ui.panelMenu;
const Lang = imports.lang;
const Soup = imports.gi.Soup;
const Mainloop = imports.mainloop;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

const SETTINGS_SCHEMA = "org.gnome.shell.extensions.icinga-checker";
let settings;
settings = Convenience.getSettings(SETTINGS_SCHEMA);

const username = settings.get_string("username");
const password = settings.get_string("password");
const urltype = (settings.get_int("urltype")==0) ? "http://" : "https://";
const urlcgi = settings.get_string("urlcgi");
const timeoutIntervalID = settings.get_int("timeoutinterval");

let timeoutInterval;

switch (timeoutIntervalID) {
    case 0:
        timeoutInterval = 30000;
        break;
    case 1:
        timeoutInterval = 60000;
        break;
    case 2:
        timeoutInterval = 90000;
        break;
    case 3:
        timeoutInterval = 120000;
        break;
    case 4:
        timeoutInterval = 150000;
        break;
    case 5:
        timeoutInterval = 180000;
        break;
    case 6:
        timeoutInterval = 300000;
        break;
    case 7:
        timeoutInterval = 600000;
        break;
    default:
        timeoutInterval = 60000;
        break;
}

let url1 = urltype+username+":"+password+"@"+urlcgi+"/status.cgi?host=all&style=hostdetail&hostprops=10&hoststatustypes=12&limit=0&start=1&csvoutput";
let url2 = urltype+username+":"+password+"@"+urlcgi+"/status.cgi?host=all&style=detail&serviceprops=10&servicestatustypes=28&limit=0&start=1&csvoutput";

var _httpSession = new Soup.SessionAsync();
Soup.Session.prototype.add_feature.call(_httpSession, new Soup.ProxyResolverDefault());
var request1 = Soup.Message.new('GET', url1);
var request2 = Soup.Message.new('GET', url2);

function MyPopupMenuItem() {
    this._init.apply(this, arguments);
}

MyPopupMenuItem.prototype = {
    __proto__: PopupMenu.PopupBaseMenuItem.prototype,

    _init: function(gicon, text1,text2,text3,text4,text5,row,klasa, params) {
        PopupMenu.PopupBaseMenuItem.prototype._init.call(this, params);
        
        if (gicon) {
            this.box = new St.BoxLayout({ style_class: 'popup-combobox-item' });
            this.icon = new St.Icon({ icon_name: gicon, style_class: 'popup-menu-icon' });
            this.box.add(this.icon);
            this.label = new St.Label({ text: text1 });
            this.box.add(this.label);
            this.actor.add(this.box);
        }
        else {
            this.table = new St.Widget({style_class: 'table',
                                       layout_manager: new Clutter.TableLayout(),
                                       reactive: true});
            
            this.label = new St.Label({ text: " "+text1+" ", style_class: klasa });
            this.table.layout_manager.pack(this.label, 0, row);
            
            this.label = new St.Label({ text: " "+text2+" ", style_class: klasa });
            this.table.layout_manager.pack(this.label, 1, row);
            
            this.label = new St.Label({ text: " "+text3+" ", style_class: klasa+"_lite" });
            this.table.layout_manager.pack(this.label, 2, row);
            
            this.label = new St.Label({ text: " "+text4+" ", style_class: klasa+"_lite" });
            this.table.layout_manager.pack(this.label, 3, row);
            
            this.label = new St.Label({ text: " "+text5+" ", style_class: klasa+"_lite" });
            this.table.layout_manager.pack(this.label, 4, row);
            
            //this.box.add(this.table);
            this.actor.add(this.table);
        }
        
        /*
        this.box = new St.BoxLayout({ style_class: 'popup-combobox-item' });
        
        if (gicon)
            this.icon = new St.Icon({ gicon: gicon, style_class: 'popup-menu-icon' });
        else
            this.icon = new St.Icon({ icon_name: 'edit-clear-symbolic', icon_size: 22 });
        
        this.box.add(this.icon);
        this.label = new St.Label({ text: text });
        this.box.add(this.label);
        this.actor.add(this.box);
        */
    },
};

function IcingaChecker() {
    this._init.apply(this, arguments);
}

IcingaChecker.prototype = {
    __proto__: PanelMenu.Button.prototype,
 
    _init: function() {
        PanelMenu.Button.prototype._init.call(this, 0.0);
        this.connect('destroy', Lang.bind(this, this._onDestroy));
        
        this._display();
        
        Main.panel.addToStatusArea('recent-items', this);
    },
    
    _onDestroy: function() {
        
    },
    
   _display: function() {
        
        //let menuItem = new MyPopupMenuItem('view-refresh-symbolic', '',false,false,false,false,false, {});
        //this.menu.addMenuItem(menuItem);
        
        if (this.timeout) {
            //log("Kasujemy mainloop od id "+this.timeout);
            Mainloop.source_remove(this.timeout);
        }
        
        //menuItem.connect('activate', Lang.bind(this, this._redisplay));
        
        this._actions = new PopupMenu.PopupBaseMenuItem({
            reactive: false,
            can_focus: false
        });
        
        this.refreshActor = new St.Button({
            reactive: true,
            can_focus: true,
            track_hover: true,
            style_class: 'system-menu-action'
        });
    
        this.refreshActor.child = new St.Icon({ icon_name: 'view-refresh-symbolic' });
        this._actions.actor.add(this.refreshActor, { expand: true, x_fill: false });
        this.menu.addMenuItem(this._actions, 100);
        this.refreshActor.connect('clicked', Lang.bind(this, this._redisplay));
        
        GetDataFromIcinga(this);
    },
    _redisplay: function() {
        this.menu.removeAll();
        this._display();
    },
    _launchFile: function(a, b, c) {
        Gio.app_info_launch_default_for_uri(c, global.create_app_launch_context(0, -1));
    },
    _clearAll: function() {
        
    },
    
    _onNetworkStateChanged: function() {
        this._checkConnectionState();
    },

    _checkConnectionState: function() {
        let address = Gio.NetworkAddress.parse_uri(url, 80);
        let cancellable = Gio.Cancellable.new();
        this._oldConnected = this._connected;
        this._connected = false;
        try {
            this._network_monitor.can_reach_async(address, cancellable, Lang.bind(this, this._asyncReadyCallback));
        } catch (err) {
            let title = _("Can not connect to %s").format(url);
            log(title + '\n' + err.message);
        }
    },
};

function init() {
    
}

function GetDataFromIcinga(id,hostslines,serviceslines) {
    
    if (!hostslines) {
        _httpSession.queue_message(request1, function(_httpSession, message) {
            let lines;
            var str;
            str = message.response_body.data;
            lines = str.split("\n");
            GetDataFromIcinga(id,lines);
        });
    }
    else if (!serviceslines) {
        _httpSession.queue_message(request2, function(_httpSession, message) {
            let lines;
            var str;
            str = message.response_body.data;
            lines = str.split("\n");
            GetDataFromIcinga(id,hostslines,lines);
        });
    }
    else {
        DisplayDataFromIcinga(id,hostslines,serviceslines);
    }
}

function DisplayDataFromIcinga(id,hostslines,serviceslines) {
    var str;
    var CountOfHostLines;
    var CountOfServicesLines;
    var HostsDown = [];
    var HostsUnreachable = [];
    var ServicesWarning = [];
    var ServicesCritical = [];
    var CountOfHostsDown;
    var CountOfHostsUnreachable;
    var CountOfServicesWarning;
    var CountOfServicesCritical;
    
    CountOfHostLines = hostslines.length;
    CountOfServicesLines = serviceslines.length;
    
    var i;
    for(i=1;i<CountOfHostLines;i++) {
        if (!hostslines[i]) {
            continue;
        }
        else {
            var data;
            data = hostslines[i].split("';'");
            if (data[1]=="DOWN") {
                HostsDown.push(data);
            }
            else {
                HostsUnreachable.push(data);
            }
        }
    }
    var i;
    for(i=1;i<CountOfServicesLines;i++) {
        if (!serviceslines[i]) {
            continue;
        }
        else {
            var data;
            data = serviceslines[i].split("';'");
            if (data[2]=="WARNING") {
                ServicesWarning.push(data);
            }
            else {
                ServicesCritical.push(data);
            }
        }
    }
    
    CountOfHostsDown = HostsDown.length;
    CountOfHostsUnreachable = HostsUnreachable.length;
    CountOfServicesWarning = ServicesWarning.length;
    CountOfServicesCritical = ServicesCritical.length;
    
    let klasa;
    if (CountOfHostsDown>0 || CountOfServicesCritical>0)
        klasa = "critical";
    else if (CountOfHostsUnreachable>0)
        klasa = "unreachable";
    else if (CountOfServicesWarning>0)
        klasa = "warning";
    else
        klasa = "labelOnPanel";
    
    id._statusLabel = new St.Label({ text: CountOfHostsDown+" | "+CountOfHostsUnreachable+" | "+CountOfServicesWarning+" | "+CountOfServicesCritical, style_class: klasa, y_align: Clutter.ActorAlign.CENTER});
    if (id.actor.first_child)
        id.actor.remove_child(id.actor.first_child);
    id.actor.add_actor(id._statusLabel);
    
    var i;
    var row=0;
    
    for(i=0;i<CountOfHostsDown;i++) {
        let menuItem = new MyPopupMenuItem(false, HostsDown[i][0].substr(1),HostsDown[i][1],HostsDown[i][3],HostsDown[i][4],HostsDown[i][5],row,"critical", {});
        id.menu.addMenuItem(menuItem);
        let url = urltype+urlcgi+"extinfo.cgi?type=1&host="+HostsDown[i][0].substr(1);
        menuItem.connect('activate', Lang.bind(this, function() {
            Gtk.show_uri(null, url, global.get_current_time());
        }));
        row++;
    }
    for(i=0;i<CountOfHostsUnreachable;i++) {
        let menuItem = new MyPopupMenuItem(false, HostsUnreachable[i][0].substr(1),HostsUnreachable[i][1],HostsUnreachable[i][3],HostsUnreachable[i][4],HostsUnreachable[i][5],row,"unreachable", {});
        id.menu.addMenuItem(menuItem);
        let url = urltype+urlcgi+"extinfo.cgi?type=1&host="+HostsUnreachable[i][0].substr(1);
        menuItem.connect('activate', Lang.bind(this, function() {
            Gtk.show_uri(null, url, global.get_current_time());
        }));
        row++;
    }
    for(i=0;i<CountOfServicesWarning;i++) {
        let menuItem = new MyPopupMenuItem(false, ServicesWarning[i][0].substr(1),ServicesWarning[i][2],ServicesWarning[i][4],ServicesWarning[i][5],ServicesWarning[i][6],row,"warning", {});
        id.menu.addMenuItem(menuItem);
        let url = urltype+urlcgi+"extinfo.cgi?type=2&host="+ServicesWarning[i][0].substr(1)+"&service="+ServicesWarning[i][1];
        menuItem.connect('activate', Lang.bind(this, function() {
            Gtk.show_uri(null, url, global.get_current_time());
        }));
        row++;
    }
    for(i=0;i<CountOfServicesCritical;i++) {
        let menuItem = new MyPopupMenuItem(false, ServicesCritical[i][0].substr(1),ServicesCritical[i][2],ServicesCritical[i][4],ServicesCritical[i][5],ServicesCritical[i][6],row,"critical", {});
        id.menu.addMenuItem(menuItem);
        let url = urltype+urlcgi+"extinfo.cgi?type=2&host="+ServicesCritical[i][0].substr(1)+"&service="+ServicesCritical[i][1];
        menuItem.connect('activate', Lang.bind(this, function() {
            Gtk.show_uri(null, url, global.get_current_time());
        }));
        row++;
    }
    
    id.timeout = Mainloop.timeout_add(timeoutInterval, function () { Icinga._redisplay(); });
}

let Icinga;

function enable() {
    Icinga = new IcingaChecker();
}
function disable() {
    Icinga.destroy();
}
